package corp.wattson.plinstool.configuration;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.HierarchicalConfiguration;
import org.apache.commons.configuration.XMLConfiguration;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import corp.wattson.plinstool.core.AbstractItem;
import corp.wattson.plinstool.core.ItemFactory;
import corp.wattson.plinstool.core.Option;
import corp.wattson.plinstool.core.Platform;
import corp.wattson.plinstool.core.SoftwareDefinition;

public class PlatformReader {

  static final String MISSING_DESCRIPTION = "No description found!";

  static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";

  static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";

  static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";

  static final String schemaSource = PlatformReader.class.getResource("/conf/platform_def.xsd").getPath();

  /**
   * 
   * @param is
   * @return
   */
  public Platform readPlatform(File platformFile) throws PlatformConfigurationException {

    XMLConfiguration xmlConfig;
    try {
      validateAgainstSchema(platformFile);
      xmlConfig = new XMLConfiguration(platformFile);
      return createPlatform(xmlConfig);
    }
    catch (ConfigurationException e) {
      throw new PlatformConfigurationException("Error reading the platform specification file.", e);
    }
    catch (ParserConfigurationException e) {
      throw new PlatformConfigurationException(
          "Unable to create the document builder to validate the platform configuration file!", e);
    }
    catch (SAXException e) {
      throw new PlatformConfigurationException("The configuration file is not valid!", e);
    }
    catch (IOException e) {
      throw new PlatformConfigurationException("The configuration file is not valid!", e);
    }
  }

  private static void validateAgainstSchema(File platformConfig) throws ParserConfigurationException, SAXException,
      IOException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
    factory.setAttribute(JAXP_SCHEMA_SOURCE, new File(schemaSource));
    factory.setNamespaceAware(true);
    factory.setValidating(true);
    DocumentBuilder builder = factory.newDocumentBuilder();
    builder.setErrorHandler(new ErrorHandler() {
      
      @Override
      public void warning(SAXParseException exception) throws SAXException {
        System.err.println(exception.getMessage());
      }
      
      @Override
      public void fatalError(SAXParseException exception) throws SAXException {
        throw exception;
      }
      
      @Override
      public void error(SAXParseException exception) throws SAXException {
        throw exception; 
      }
    });
    builder.parse(platformConfig);
  }

  /**
   * 
   * @param xmlconfig
   * @return
   * @throws PlatformConfigurationException
   */
  private Platform createPlatform(XMLConfiguration xmlconfig) throws PlatformConfigurationException {
    Platform platform = new Platform();
    platform.setName(xmlconfig.getString("Platform[@name]"));
    platform.setDescription(xmlconfig.getString("Platform.description", MISSING_DESCRIPTION));
    // Software
    ArrayList<SoftwareDefinition> softwareList = new ArrayList<SoftwareDefinition>();
    HashMap<String, SoftwareDefinition> softwareMap = new HashMap<String, SoftwareDefinition>();
    HashMap<String, ArrayList<String>> softDependenciesMap = new HashMap<String, ArrayList<String>>();

    Object softNameList = xmlconfig.getProperty("Platform.software[@name]");
    if (softNameList instanceof Collection<?>) {
      for (int i = 0; i < ((Collection<?>) softNameList).size(); i++) {
        HierarchicalConfiguration subconfig = xmlconfig.configurationAt("Platform.software(" + i + ")");
        String name = subconfig.getString("[@name]");
        // Prepare the dependencies
        softDependenciesMap.put(name, new ArrayList<String>());

        SoftwareDefinition software = new SoftwareDefinition();
        software.setName(name);
        software.setCompliancyLevel(SoftwareDefinition.Level.valueOf(subconfig.getString("[@level]", "MANDATORY")
            .toUpperCase(Locale.ENGLISH)));
        software.setDescription(subconfig.getString("description", MISSING_DESCRIPTION));

        // Manage items
        ArrayList<AbstractItem> items = new ArrayList<AbstractItem>();
        Object itemsNameList = subconfig.getProperty("item[@name]");
        if (itemsNameList instanceof Collection<?>) {
          for (int j = 0; j < ((Collection<?>) itemsNameList).size(); j++) {
            parseItem(subconfig, items, j);
          }
        }
        else if (itemsNameList instanceof String) {
          parseItem(subconfig, items, 0);
        }

        // Manage dependencies
        ArrayList<String> dependencies = softDependenciesMap.get(name);
        Object depNameList = subconfig.getProperty("dependency[@name]");
        if (depNameList instanceof Collection<?>) {
          for (int j = 0; j < ((Collection<?>) depNameList).size(); j++) {
            dependencies.add(subconfig.getString("dependency(" + j + ")[@name]"));
          }
        }
        else if (depNameList instanceof String) {
          dependencies.add((String) depNameList);
        }

        software.setItems(items);
        softwareList.add(software);
        softwareMap.put(name, software);
      }
    }

    // Manage dependencies after all softwares are done.
    for (SoftwareDefinition software : softwareList) {
      String name = software.getName();
      ArrayList<SoftwareDefinition> dependencies = new ArrayList<SoftwareDefinition>();
      for (String dependency : softDependenciesMap.get(name)) {
        SoftwareDefinition dependencySoft = softwareMap.get(dependency);
        // Missing dependency
        if (dependencySoft == null) {
          throw new PlatformConfigurationException("Some specified dependencies are missing: \n" + "\t - software: "
              + name + "\n" + "\t - missing dependency: " + dependency);
        }
        dependencies.add(dependencySoft);
      }
      software.setDependencies(dependencies);
    }

    // check for dependencies loop
    checkDependenciesLoop(softwareList);

    platform.setSoftwares(softwareList);
    return platform;
  }

  /**
   * 
   * @param softwareList
   * @throws PlatformConfigurationException
   */
  private void checkDependenciesLoop(ArrayList<SoftwareDefinition> softwareList) throws PlatformConfigurationException {
    for (SoftwareDefinition software : softwareList) {
      checkForLoop(software, software.getName());
    }
  }

  /**
   * 
   * @param software
   * @param name
   * @throws PlatformConfigurationException if a loop is found.
   */
  private void checkForLoop(SoftwareDefinition software, String name) throws PlatformConfigurationException {
    for (SoftwareDefinition dependency : software.getDependencies()) {
      if (name.equalsIgnoreCase(dependency.getName())) {
        throw new PlatformConfigurationException("Dependency loop found in the platform definition: software=" + name);
      }
      checkForLoop(dependency, name);
    }
  }

  /**
   * 
   * @param subconfig
   * @param items
   * @param i
   */
  private void parseItem(HierarchicalConfiguration subconfig, ArrayList<AbstractItem> items, int i) {
    String itemNode = "item(" + i + ")";
    AbstractItem item = ItemFactory.getItem(subconfig.getString(itemNode + "[@type]"));
    item.setName(subconfig.getString(itemNode + "[@name]"));
    item.setDescription(subconfig.getString(itemNode + ".description", MISSING_DESCRIPTION));

    // manage options
    ArrayList<Option> options = new ArrayList<Option>();
    Object optionsName = subconfig.getProperty(itemNode + ".option[@name]");
    if (optionsName != null) {
      if (optionsName instanceof String) {
        parseOption(subconfig, options, itemNode + ".option");
      }
      else if (optionsName instanceof Collection<?>) {
        for (int j = 0; j < ((Collection<?>) optionsName).size(); j++) {
          parseOption(subconfig, options, itemNode + ".option(" + j + ")");
        }
      }
    }
    item.setOptionList(options);
    items.add(item);
  }

  /**
   * 
   * @param subconfig
   * @param options
   * @param optionNode
   */
  private void parseOption(HierarchicalConfiguration subconfig, ArrayList<Option> options, String optionNode) {
    Option option = new Option();
    option.setName(subconfig.getString(optionNode + "[@name]"));
    option.setValue(subconfig.getString(optionNode + "[@value]"));
    options.add(option);
  }

}
