package corp.wattson.plinstool.core;

import java.net.URL;

public class ItemFactory {

  /**
   * Runtime exception sent when an Item is not able to be created. This exception will always embed
   * a causing exception providing more information.
   * 
   * @author wattazoum
   * 
   */
  public static class ItemCreationException extends RuntimeException {

    private static final long serialVersionUID = -4884755164469945062L;

    /**
     * Only allowed constructor. We need to always get the real cause of the exception.
     * 
     * @param message A message describing the problem.
     * @param e The cause of the problem.
     */
    public ItemCreationException(String message, Throwable e) {
      super(message, e);
    }

  }

  private static final String COMMON_ITEMS_PACKAGES = "corp.wattson.plinstool.items.common.";

  private static String osItemsPackage = "corp.wattson.plinstool.items.";
  static {
    String osName = System.getProperty("os.name");
    osName = osName.split(" ")[0].toLowerCase();
    osItemsPackage += osName + ".";
  }

  private static String customItemsPackage;

  /**
   * Get an instance of the {@link AbstractItem} implementation corresponding to the given type.
   * 
   * @param type The type of the {@link AbstractItem} to retrieve.
   * @return An instance of the {@link AbstractItem} implementation corresponding to the type.
   * @throws ItemCreationException if there is a problem getting the AbstractItem
   */
  public static AbstractItem getItem(String type) throws ItemCreationException {

    if (type != null && type.length() > 0) {
      String classPrefix;
      if (type.length() == 1) {
        classPrefix = type.toUpperCase();
      }
      else {
        char[] chars = type.toCharArray();
        chars[0] = Character.toUpperCase(chars[0]);
        classPrefix = new String(chars);
      }

      try {
        String className = findTheClass(classPrefix + "Item");
        try {
          AbstractItem item = instanciate(className);
          item.setType(type);
          return item;
        }
        catch (ClassCastException e) {
          throw new ItemCreationException("The class '" + className +"' is not an AbstractItem.", e);
        }
        catch (ClassNotFoundException e) {
          //This should never happen. The findTheClass method prevents it 
          throw new ItemCreationException("Unable to find class: " + className, e);
        }
        catch (InstantiationException e) {
          throw new ItemCreationException("Unable to instanciate class: " + className, e);
        }
        catch (IllegalAccessException e) {
          throw new ItemCreationException("The default constructor is not accessible: " + className, e);
        }
      }
      catch (ClassNotFoundException e) {
        throw new ItemCreationException("Unable to find class for item type: " + type, e);
      }
    }
    else {
      throw new IllegalArgumentException("The type shouldn't be null, nor empty !");
    }
  }

  /**
   * 
   * @param className
   * @return
   * @throws ClassNotFoundException
   * @throws InstantiationException
   * @throws IllegalAccessException
   * @throws ClassCastException If the className doesn't denote an AbstractItem.
   */
  private static AbstractItem instanciate(String className) throws ClassNotFoundException, InstantiationException,
      IllegalAccessException, ClassCastException {
    Class<?> itemClass = Class.forName(className);
    return AbstractItem.class.cast(itemClass.newInstance());
  }

  /**
   * First search in the custom package, if not null. Then search in the OS package and fall back in
   * common package.
   * 
   * @param className
   * 
   * @return The className to use to load the class.
   * @throws ClassNotFoundException If the file is not found in all packages.
   */
  private static String findTheClass(String className) throws ClassNotFoundException {
    if (customItemsPackage != null) {
      if (checkForResourceName(customItemsPackage, className)) {
        return customItemsPackage + className;
      }
    }
    // We didn't find it, search in osPackage
    if (checkForResourceName(osItemsPackage, className)) {
      return osItemsPackage + className;
    }
    else {
      // Now common package
      if (checkForResourceName(COMMON_ITEMS_PACKAGES, className)) {
        return COMMON_ITEMS_PACKAGES + className;
      }
      else {
        StringBuilder error = new StringBuilder("The class '").append(className).append("' ");
        error.append("was not found in the packages:\n");
        if (customItemsPackage != null) {
          error.append("- ").append(customItemsPackage).append("\n");
        }
        error.append("- ").append(osItemsPackage).append("\n");
        error.append("- ").append(COMMON_ITEMS_PACKAGES).append("\n");
        throw new ClassNotFoundException(error.toString());
      }
    }
  }

  private static boolean checkForResourceName(String packageToCheck, String className) {
    StringBuilder temporaryResource = new StringBuilder("/");
    temporaryResource.append(packageToCheck.replace('.', '/'));
    temporaryResource.append(className).append(".class");
    URL classURL = ItemFactory.class.getResource(temporaryResource.toString());
    return classURL != null;
  }

  /**
   * Set a custom package were to search Item from (eg. <code>my.custom.items.pkg</code>). If the
   * customPackage parameter is set to <code>null</code>, we drop the custom package (no custom
   * package will be used).
   * 
   * @param customPackage the custom package to set.
   */
  static synchronized void setCustomPackage(String customPackage) {
    customItemsPackage = customPackage != null ? customPackage + "." : null;
  }

}
