package corp.wattson.plinstool.core;

import java.util.ArrayList;
import java.util.List;

/**
 * Schema fragment(s) for this class:
 * 
 * <pre>
 * &lt;xs:element xmlns:ns="http://plinstool.wattazoum.fr/platform_def/" xmlns:xs="http://www.w3.org/2001/XMLSchema" name="platform">
 *   &lt;xs:complexType>
 *     &lt;xs:sequence>
 *       &lt;xs:element type="xs:string" name="description"/>
 *       &lt;xs:element name="software" minOccurs="0" maxOccurs="unbounded">
 *         &lt;!-- Reference to inner class Software -->
 *       &lt;/xs:element>
 *     &lt;/xs:sequence>
 *     &lt;xs:attribute type="xs:string" name="name"/>
 *   &lt;/xs:complexType>
 * &lt;/xs:element>
 * </pre>
 */
public class Platform implements Describable, Processable {
  private String name;
  private String description;
  private ProcessableState state = ProcessableState.NOT_CHECKED;
  private List<SoftwareDefinition> softwareList = new ArrayList<SoftwareDefinition>();

  /**
   * Get the 'description' element value.
   * 
   * @return value
   */
  @Override
  public String describe() {
    return description;
  }

  /**
   * Set the 'description' element value.
   * 
   * @param description
   */
  public void setDescription(String description) {
    this.description = description;
  }

  /**
   * Get the list of 'software' element items.
   * 
   * @return list
   */
  public List<SoftwareDefinition> getSoftwares() {
    return softwareList;
  }

  /**
   * Set the list of 'software' element items.
   * 
   * @param list
   */
  public void setSoftwares(List<SoftwareDefinition> list) {
    softwareList = list;
  }

  /**
   * Get the 'name' attribute value.
   * 
   * @return value
   */
  public String getName() {
    return name;
  }

  /**
   * Set the 'name' attribute value.
   * 
   * @param name
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * @throws NotCompliantInstallException
   * 
   */
  public void check() throws NotCompliantInstallException {
    boolean brokenFound = false;

    StringBuilder stateMessage = new StringBuilder();
    for (SoftwareDefinition software : softwareList) {
      try {
        software.check();
      }
      catch (NotCompliantInstallException e) {
        brokenFound = true;
        stateMessage.append("- ").append(software.getName()).append(" is broken: ");
        stateMessage.append(e.getMessage()).append("\n");
      }
    }

    if (brokenFound) {
      state = ProcessableState.BROKEN;
      state.setMessage(stateMessage.toString());
      throw new NotCompliantInstallException(stateMessage.toString());
    }
    else {
      state = ProcessableState.FINE;
    }
  }

  /**
   * @throws ProcessFailedException
   * 
   */
  @Override
  public void process() throws ProcessFailedException {
    for (SoftwareDefinition software : softwareList) {
      software.process();
    }
  }

  /**
   * The state depends on the state of each software. If we find one Broken state, the platform is
   * broken. If we find one NOT_CHECKED state, the platform is considered not checked. If none of
   * them is found, the platform is fine.
   */
  @Override
  public ProcessableState getState() {

    if (state == ProcessableState.NOT_CHECKED) {
      return state;
    }
    else {
      StringBuilder stateMessage = new StringBuilder();

      boolean brokenFound = false;
      boolean notCheckedFound = false;
      for (SoftwareDefinition software : softwareList) {
        switch (software.getState()) {
          case BROKEN:
            brokenFound = true;
            stateMessage.append("- ").append(software.getName()).append(" is broken!\n");
            break;
          case NOT_CHECKED:
            notCheckedFound = true;
            stateMessage.append("- ").append(software.getName()).append(" is not checked!");
            break;
          default:
            break;
        }
      }

      // Prefer Broken state to NotChecked state
      if (brokenFound) {
        state = ProcessableState.BROKEN;
        state.setMessage(stateMessage.toString());
      }
      else if (notCheckedFound) {
        state = ProcessableState.NOT_CHECKED;
        state.setMessage(stateMessage.toString());
      }
      else {
        state = ProcessableState.FINE;
      }

      return state;
    }
  }

  @Override
  public List<Describable> getIncludedDescribable() {
    ArrayList<Describable> result = new ArrayList<Describable>(softwareList);
    return result;
  }

}
