package corp.wattson.plinstool.core;

import java.util.ArrayList;
import java.util.List;

public class SoftwareDefinition implements Describable, Processable {

  public enum Level {
    MANDATORY, RECOMMENDED, OPTIONAL;
  }

  private String name;
  private String description;
  private Level compliancyLevel = Level.MANDATORY;
  private Describable.ProcessableState state = Describable.ProcessableState.NOT_CHECKED;
  private List<AbstractItem> items = new ArrayList<AbstractItem>();
  private List<SoftwareDefinition> dependencies = new ArrayList<SoftwareDefinition>();

  /**
   * @return the name
   */
  @Override
  public String getName() {
    return name;
  }

  
  /**
   * {@inheritDoc}.
   */
  @Override
  public String describe() {
    return description;
  }

  /**
   * {@inheritDoc}.
   */
  @Override
  public void check() throws NotCompliantInstallException {
    try {
      for (Processable item : dependencies) {
        item.check();
      }
      for (Processable item : items) {
        item.check();
      }
      state = Describable.ProcessableState.FINE;
    }
    catch (NotCompliantInstallException e) {
      state = Describable.ProcessableState.BROKEN;
      throw e;
    }
  }
  
  /**
   * Returns the state of this describable. 
   */
  @Override
  public Describable.ProcessableState getState() {
    return state;
  }

  /**
   * {@inheritDoc}.
   */
  @Override
  public void process() throws ProcessFailedException {
    // First process dependencies
    for (SoftwareDefinition dependency : dependencies) {
      dependency.process();
    }

    // Then process the items.
    for (Processable item : items) {
      item.process();
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public List<Describable> getIncludedDescribable() {
    ArrayList<Describable> result = new ArrayList<Describable>();
    result.addAll(items);
    result.addAll(dependencies);
    return result;
  }
  
  public List<SoftwareDefinition> getDependencies() {
    return dependencies;
  }

  public void setDependencies(ArrayList<SoftwareDefinition> dependencies) {
    this.dependencies = dependencies;
  }

  /**
   * 
   * @return
   */
  public List<AbstractItem> getItems() {
    return items;
  }

  public void setItems(List<AbstractItem> items) {
    this.items = items;
  }

  /**
   * @param name the name to set
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * @param compliancyLevel the compliancyLevel to set
   */
  public void setCompliancyLevel(Level compliancyLevel) {
    this.compliancyLevel = compliancyLevel;
  }

  /**
   * @return the compliancyLevel
   */
  public Level getCompliancyLevel() {
    return compliancyLevel;
  }

  public void setDescription(String description) {
    this.description = description;
  }

}
