package corp.wattson.plinstool.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.net.URL;

import org.junit.Test;
import org.xml.sax.SAXParseException;

import corp.wattson.plinstool.core.AbstractItem;
import corp.wattson.plinstool.core.Platform;
import corp.wattson.plinstool.core.SoftwareDefinition;
import corp.wattson.plinstool.core.SoftwareDefinition.Level;

public class PlatformReaderTest {

  @Test
  public void testReadPlatform() throws PlatformConfigurationException {
    URL config = PlatformReaderTest.class.getResource("platform_example.xml");
    PlatformReader reader = new PlatformReader();
    Platform platform = reader.readPlatform(new File(config.getPath()));
    assertEquals("Testing Platform", platform.getName());
    assertEquals("My Test Platform", platform.describe());
    assertNotNull(platform.getSoftwares());
    assertEquals(2, platform.getSoftwares().size());

    SoftwareDefinition soft = platform.getSoftwares().get(0);
    assertEquals("Eclipse 3.5", soft.getName());
    assertEquals(Level.MANDATORY, soft.getCompliancyLevel());
    assertEquals("Eclipse IDE", soft.describe());
    assertNotNull(soft.getItems());
    assertEquals(3, soft.getItems().size());
    assertNotNull(soft.getDependencies());
    assertEquals(1, soft.getDependencies().size());
    assertEquals("JDK 6", soft.getDependencies().get(0).getName());

    // test on Items
    AbstractItem item = soft.getItems().get(0);
    assertEquals("ide", item.getName());
    assertEquals("zip", item.getType());
    assertNotNull(item.describe());
    assertEquals(PlatformReader.MISSING_DESCRIPTION, item.describe());
    assertNotNull(item.getOptions());
    assertEquals(2, item.getOptions().size());

    item = soft.getItems().get(2);
    assertNotNull(item.describe());

    // Test the second software
    soft = platform.getSoftwares().get(1);
    assertEquals("JDK 6", soft.getName());
    assertEquals(Level.MANDATORY, soft.getCompliancyLevel());
    assertNotNull(soft.describe());
    assertEquals(PlatformReader.MISSING_DESCRIPTION, soft.describe());
    assertNotNull(soft.getItems());
    assertEquals(2, soft.getItems().size());
    assertNotNull(soft.getDependencies());
    assertEquals(0, soft.getDependencies().size());
  }

  @Test
  public void testEmptyPlatform() throws PlatformConfigurationException {
    PlatformReader reader = new PlatformReader();
    URL url = PlatformReaderTest.class.getResource("platform_nosoft.xml");
    Platform platform = reader.readPlatform(new File(url.getPath()));
    assertNotNull(platform);
    assertNotNull(platform.describe());
    assertNotSame(PlatformReader.MISSING_DESCRIPTION, platform.describe());
    assertNotNull(platform.getSoftwares());
    assertEquals(0, platform.getSoftwares().size());
  }

  @Test
  public void testNoDescPlatform() throws PlatformConfigurationException {
    PlatformReader reader = new PlatformReader();
    URL url = PlatformReaderTest.class.getResource("platform_nodesc.xml");
    Platform platform = reader.readPlatform(new File(url.getPath()));
    assertNotNull(platform);
    assertNotNull(platform.describe());
    assertEquals(PlatformReader.MISSING_DESCRIPTION, platform.describe());
    assertNotNull(platform.getSoftwares());
    assertEquals(2, platform.getSoftwares().size());

    assertNotNull(platform.getSoftwares().get(0));
    assertEquals(PlatformReader.MISSING_DESCRIPTION, platform.getSoftwares().get(0).describe());
    assertNotNull(platform.getSoftwares().get(1));
    assertEquals(PlatformReader.MISSING_DESCRIPTION, platform.getSoftwares().get(1).describe());
  }

  @Test
  public void testMultiDependencies() throws PlatformConfigurationException {
    PlatformReader reader = new PlatformReader();
    URL url = PlatformReaderTest.class.getResource("platform_multipledeps.xml");
    Platform platform = reader.readPlatform(new File(url.getPath()));
    assertNotNull(platform);
    assertEquals(2, platform.getSoftwares().get(0).getDependencies().size());
  }

  @Test
  public void testPlatformDepLoop() {
    PlatformReader reader = new PlatformReader();
    try {
      URL url = PlatformReaderTest.class.getResource("platform_deploop.xml");
      reader.readPlatform(new File(url.getPath()));
      fail("The definition file should be invalid: dependency loop.");
    }
    catch (PlatformConfigurationException e) {
      assertEquals("Dependency loop found in the platform definition: software=Eclipse 3.5", e.getMessage());
    }
    try {
      URL url = PlatformReaderTest.class.getResource("platform_depsloop_depth3.xml");
      reader.readPlatform(new File(url.getPath()));
      fail("The definition file should be invalid: dependency loop.");
    }
    catch (PlatformConfigurationException e) {
      assertEquals("Dependency loop found in the platform definition: software=Eclipse 3.5", e.getMessage());
    }
  }

  @Test
  public void testInvalidXMLFile() {
    PlatformReader reader = new PlatformReader();
    try {
      URL url = PlatformReaderTest.class.getResource("platform_invalidxml.xml");
      reader.readPlatform(new File(url.getPath()));
      fail("The definition file should be invalid: invalid xml.");
    }
    catch (PlatformConfigurationException e) {
      assertNotNull(e.getCause());
      assertTrue("The cause is SAXParserException", e.getCause() instanceof SAXParseException);
    }
  }

  @Test
  public void testMissingDependency() {
    PlatformReader reader = new PlatformReader();
    try {
      URL url = PlatformReaderTest.class.getResource("platform_missingdeps.xml");
      reader.readPlatform(new File(url.getPath()));
      fail("The definition file should be invalid: Missing dependency.");
    }
    catch (PlatformConfigurationException e) {
      assertNotNull(e.getMessage());
      assertTrue(e.getMessage().startsWith("Some specified dependencies are missing: "));
    }
  }

  @Test
  public void testNotCompliantXML() {
    PlatformReader reader = new PlatformReader();
    try {
      URL url = PlatformReaderTest.class.getResource("platform_notcompliantxml.xml");
      reader.readPlatform(new File(url.getPath()));
      fail("The definition file should be invalid. Not compliant with the schema");
    }
    catch (PlatformConfigurationException e) {
      assertNotNull(e.getCause());
      assertTrue("The cause is a SAXParseException", e.getCause() instanceof SAXParseException);
    }
  }

}
